//************************************************************************
// Hiddll.cpp
//
// Get function adresspointers from HID.DLL by name.
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 31.12.2003
// Compiler Visual C++ V5.0
//************************************************************************
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "setupapidll.h"
#include "hiddll.h"
#include "picio.h"

DLL_HidD_GetHidGuid HidD_GetHidGuid;
DLL_HidD_GetAttributes HidD_GetAttributes;
DLL_HidD_GetFeature HidD_GetFeature;
DLL_HidD_SetFeature HidD_SetFeature;
DLL_HidD_GetNumInputBuffers HidD_GetNumInputBuffers;
DLL_HidD_SetNumInputBuffers HidD_SetNumInputBuffers;
DLL_HidD_GetManufacturerString HidD_GetManufacturerString;
DLL_HidD_GetProductString HidD_GetProductString;
DLL_HidD_GetSerialNumberString HidD_GetSerialNumberString;
DLL_HidD_GetPhysicalDescriptor HidD_GetPhysicalDescriptor;
DLL_HidD_GetIndexedString HidD_GetIndexedString;

HINSTANCE hhid;
HDEVINFO PnPHandle;

//#########################################
int LoadHidDll()
//#########################################
{
 hhid = LoadLibrary("hid.dll");	
 if(hhid == NULL)
  {
   MessageBox(NULL,"Error: Can't Load hid.dll",NULL,MB_OK);
   return 0;
  }
 else
  {
   //Get dll function entrys by function name
   if(!(HidD_GetHidGuid = (DLL_HidD_GetHidGuid)GetProcAddress(hhid,"HidD_GetHidGuid")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetHidGuid() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetAttributes = (DLL_HidD_GetAttributes)GetProcAddress(hhid,"HidD_GetAttributes")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetAttributes() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetManufacturerString = (DLL_HidD_GetManufacturerString)GetProcAddress(hhid,"HidD_GetManufacturerString")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetManufacturerString() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetProductString = (DLL_HidD_GetProductString)GetProcAddress(hhid,"HidD_GetProductString")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetProductString() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetSerialNumberString = (DLL_HidD_GetSerialNumberString)GetProcAddress(hhid,"HidD_GetSerialNumberString")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetSerialNumberString() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetPhysicalDescriptor = (DLL_HidD_GetPhysicalDescriptor)GetProcAddress(hhid,"HidD_GetPhysicalDescriptor")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetPhysicalDescriptor() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetIndexedString = (DLL_HidD_GetIndexedString)GetProcAddress(hhid,"HidD_GetIndexedString")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetIndexedString() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetFeature = (DLL_HidD_GetFeature)GetProcAddress(hhid,"HidD_GetFeature")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetFeature() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_SetFeature = (DLL_HidD_SetFeature)GetProcAddress(hhid,"HidD_SetFeature")))
    {
     MessageBox(NULL,"Error: Can't find HidD_SetFeature() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_GetNumInputBuffers = (DLL_HidD_GetNumInputBuffers)GetProcAddress(hhid,"HidD_GetNumInputBuffers")))
    {
     MessageBox(NULL,"Error: Can't find HidD_GetNumInputBuffers() in hid.dll",NULL,MB_OK);
     return 0;
    }

   if(!(HidD_SetNumInputBuffers = (DLL_HidD_SetNumInputBuffers)GetProcAddress(hhid,"HidD_SetNumInputBuffers")))
    {
     MessageBox(NULL,"Error: Can't find HidD_SetNumInputBuffers() in hid.dll",NULL,MB_OK);
     return 0;
    }

  }

 return 1;
}

//#########################################
void UnloadHidDll()
//#########################################
{
 if(hhid) { FreeLibrary(hhid); hhid=NULL; }
}


//#####################################################
// Searches for a hid device with known VID and PID
// returns: zero if not found
//          nozero if found and device opened
//
// Based on some VisualBasic examples from
// http://www.usb-by-example.com/
//#####################################################
int FindHIDDevice(unsigned int vid, unsigned int pid)
{
 struct _GUID HidGuid;
 struct {DWORD cbSize; char DevicePath[256];} FunctionClassDeviceData;

 SP_INTERFACE_DEVICE_DATA DeviceInterfaceData;
 _HIDD_ATTRIBUTES ThisHidDevice;

 int Success, HidDevice, i, result;
 unsigned long BytesReturned;

 char VendorName[256];
 char ProductName[256];

 char temp[2]; //for conversion UNICODE to String
 unsigned char buffer[253]; //don't use more than 253 bytes !
                           //or you can't read Vendor/Product Name from the PIC16C765

//Open HID.DLL and get function adresses by name
 result=LoadHidDll();
 if(!result) { HIDCleanup(); return 0; }
//   MessageBox(NULL,"hid.dll loaded",NULL,MB_OK);

//Open SETUPAPI.DLL and get function adresses by name
 result=LoadSetupApiDll();
 if(!result) { HIDCleanup(); return 0; }
//   MessageBox(NULL,"setupapi.dll loaded",NULL,MB_OK);

// First, get my class identifier
 HidD_GetHidGuid(&HidGuid);

 PnPHandle=SetupDiGetClassDevs(&HidGuid,0,0,DIGCF_PRESENT|DIGCF_DEVICEINTERFACE);
 if(PnPHandle != INVALID_HANDLE_VALUE)
  {

   // Lets look for a maximum of 50 HID devices
   for(HidDevice = 0; HidDevice < 50; HidDevice++)
    {
     // Initialize our Data
	 DeviceInterfaceData.cbSize = 28; // Length of data structure in bytes

     // Is there a HID device at this table entry
	 Success = SetupDiEnumDeviceInterfaces(PnPHandle, 0, &HidGuid, HidDevice, &DeviceInterfaceData);
	 if(Success == TRUE)
      {
//       MessageBox(NULL,"SetupDiEnumDeviceInterfaces Success",NULL,MB_OK);

       // There is a device here, get it's name
	   FunctionClassDeviceData.cbSize = 5;
	   Success = SetupDiGetDeviceInterfaceDetail(PnPHandle, &DeviceInterfaceData, 
	                                             PSP_INTERFACE_DEVICE_DETAIL_DATA(&FunctionClassDeviceData)
                                                 , 256, &BytesReturned, 0);
	   if (Success == FALSE)
        {
          MessageBox(NULL, "Could not find the system name for this HID device", "Error", MB_ICONSTOP);
		  HIDCleanup();
          return 0;
        }
//       MessageBox(NULL,"SetupDiGetDeviceInterfaceDetail Success",NULL,MB_OK);

       // Can now open this HID device
	   HidHandle = CreateFile(FunctionClassDeviceData.DevicePath,
                              GENERIC_READ|GENERIC_WRITE,
                              FILE_SHARE_READ|FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING,
                              FILE_ATTRIBUTE_NORMAL, NULL);
	   if(HidHandle==INVALID_HANDLE_VALUE)
        {
		 MessageBox(NULL, "Could not open HID device", "Error", MB_ICONSTOP);
 	     HIDCleanup();
         return 0;
        }
       //this looks very strange ;) 
       //sprintf(zeile,"%s",FunctionClassDeviceData.DevicePath);
       //MessageBox(NULL, zeile, "Success", MB_OK);

       // Get VID and PID
	   Success = HidD_GetAttributes(HidHandle, &ThisHidDevice);

       // Get Manufacturers name if present
	   if(HidD_GetManufacturerString(HidHandle, buffer, sizeof(buffer)))
        {
	     i = 0;
         temp[1]=0;
         strcpy(VendorName,"\0");
	     while (buffer[i] != 0) // Convert from UNICODE to String
          { temp[0]=buffer[i];
            strcat(VendorName,temp);
            i = i + 2;
          }
		} else strcpy(VendorName,"- not present -"); 

       // Get Product name if present
	   if(HidD_GetProductString(HidHandle, buffer, sizeof(buffer)))
        {
	     i = 0;
         temp[1]=0;
         strcpy(ProductName,"\0");
		 while (buffer[i] != 0) // Convert from UNICODE to String
          { temp[0]=buffer[i];
            strcat(ProductName,temp);
            i = i + 2;
          }
		} else strcpy(ProductName,"- not present -");    

//show all existing HID devices
//delete or comment it if you don't like this
       fprintf(stdout,"VID: 0x%04X PID: 0x%04X\n"
                     "VendorName: %s  ProductName: %s\n\n"
               ,ThisHidDevice.VendorID
               ,ThisHidDevice.ProductID
               ,VendorName
               ,ProductName
              );

       //is this my HID device ?
       if(ThisHidDevice.VendorID==vid && ThisHidDevice.ProductID==pid)
        {
//         fprintf(stdout,"HID device found and opened.\n");
         return 1;
        }

       //Close device if this is not what we are searching for
       if(HidHandle) { CloseHandle(HidHandle); HidHandle=NULL; }

      } // if (SetupDiEnumDeviceInterfaces . .
	} // for (HidDevice = 0; HidDevice < 50; HidDevice++)
  } //if(int(PnPHandle) != -1)

 return 0;
}

//#########################################
void HIDCleanup()
//#########################################
{
 if(HidHandle) { CloseHandle(HidHandle); HidHandle=NULL; }
 if(PnPHandle) { SetupDiDestroyDeviceInfoList(PnPHandle); PnPHandle=NULL; }
 UnloadSetupApiDll();
 UnloadHidDll();
}

